@@ -26,7 +26,7 @@ This WSGI component:
 * Collects and forwards identity information based on a valid token
   such as user name, domain, project, etc.
 
-Refer to: https://docs.openstack.org/developer/keystonemiddleware/\
+Refer to: https://docs.openstack.org/keystonemiddleware/latest/\
 middlewarearchitecture.html
 
 
@@ -227,7 +227,6 @@ from keystoneclient.common import cms
 from keystoneclient import exceptions as ksc_exceptions
 from oslo_log import log as logging
 from oslo_serialization import jsonutils
-import six
 import webob.dec
 
 from keystonemiddleware._common import config
@@ -307,10 +306,10 @@ class BaseAuthProtocol(object):
                                    perform.
     """
 
-    # NOTE(jamielennox): Default to True and remove in Pike. This is a
+    # NOTE(jamielennox): Default to True and remove in Queens. This is a
     # compatibility flag to allow passing **kwargs to fetch_token(). This
     # is basically to allow compatibility with keystone's override. We will
-    # assume all subclasses are ok with this being True in the Pike release.
+    # assume all subclasses are ok with this being True in the Queens release.
     kwargs_to_fetch_token = False
 
     def __init__(self,
@@ -431,12 +430,15 @@ class BaseAuthProtocol(object):
 
     def _do_fetch_token(self, token, **kwargs):
         """Helper method to fetch a token and convert it into an AccessInfo."""
+        # NOTE(edmondsw): strip the token to remove any whitespace that may
+        # have been passed along in the header per bug 1689468
+        token = token.strip()
         if self.kwargs_to_fetch_token:
             data = self.fetch_token(token, **kwargs)
         else:
             m = _('Implementations of auth_token must set '
                   'kwargs_to_fetch_token this will be the required and '
-                  'assumed in Pike.')
+                  'assumed in Queens.')
             warnings.warn(m)
             data = self.fetch_token(token)
 
@@ -510,7 +512,7 @@ class BaseAuthProtocol(object):
                           name)
             self._invalid_user_token()
 
-        for bind_type, identifier in six.iteritems(auth_ref.bind):
+        for bind_type, identifier in auth_ref.bind.items():
             if bind_type == _BIND_MODE.KERBEROS:
                 if req.auth_type != 'negotiate':
                     self.log.info('Kerberos credentials required and '
@@ -581,17 +583,20 @@ class AuthProtocol(BaseAuthProtocol):
         self._session = self._create_session()
         self._identity_server = self._create_identity_server()
 
-        self._auth_uri = self._conf.get('auth_uri')
-        if not self._auth_uri:
+        self._www_authenticate_uri = self._conf.get('www_authenticate_uri')
+        if not self._www_authenticate_uri:
+            self._www_authenticate_uri = self._conf.get('auth_uri')
+        if not self._www_authenticate_uri:
             self.log.warning(
-                'Configuring auth_uri to point to the public identity '
-                'endpoint is required; clients may not be able to '
+                'Configuring www_authenticate_uri to point to the public '
+                'identity endpoint is required; clients may not be able to '
                 'authenticate against an admin endpoint')
 
             # FIXME(dolph): drop support for this fallback behavior as
             # documented in bug 1207517.
 
-            self._auth_uri = self._identity_server.auth_uri
+            self._www_authenticate_uri = \
+                self._identity_server.www_authenticate_uri
 
         self._signing_directory = _signing_dir.SigningDirectory(
             directory_name=self._conf.get('signing_dir'), log=self.log)
@@ -685,7 +690,7 @@ class AuthProtocol(BaseAuthProtocol):
 
     @property
     def _reject_auth_headers(self):
-        header_val = 'Keystone uri=\'%s\'' % self._auth_uri
+        header_val = 'Keystone uri=\'%s\'' % self._www_authenticate_uri
         return [('WWW-Authenticate', header_val)]
 
     def _token_hashes(self, token):
@@ -800,6 +805,13 @@ class AuthProtocol(BaseAuthProtocol):
             self.log.warning('Fetch revocation list failed, '
                              'fallback to online validation.')
         else:
+            self.log.warning('auth_token middleware received a PKI/Z token. '
+                             'This form of token is deprecated and has been '
+                             'removed from keystone server and will be '
+                             'removed from auth_token middleware in the Rocky '
+                             'release. Please contact your administrator '
+                             'about upgrading keystone and the token format.')
+
             data = jsonutils.loads(verified)
 
             audit_ids = None
